#ifndef __NACC_OSD_H
#define __NACC_OSD_H

#include <vdr/osd.h>
#include <vdr/thread.h>


///////////////////////////////////////////////////////////////////////////////

enum captionMode { M_NONE, M_RCL, M_RDC, M_RU, M_TXT };

enum fontMode { F_REGULAR=0x01, F_ITALIC=0x02, F_UNDERLINE=0x04 };


class Scroller;


class CCOsd 
{
public:
	CCOsd();
	virtual ~CCOsd();
  
  void Show(void);
  void Hide(void);
  
  void SetPosition(int row, int column);
  void SetRow(int r) { cRow = r; }
  void CR(void);
  void TO(int i) { cCol += i; }
  void DER(void);
  void Backspace(void);
  
  bool SetMode(captionMode m, int arg = 0);
  
  void SetFontMode(fontMode fm, bool override = true);
  void AddChar(uint8_t c);

  void SetBgColor(tColor bg) { bgClr = bg; }
  void SetFgColor(tColor fg) { fgClr = fg; }
  
  void ClearScreen(void);
  void ClearBuffer(void);
  void Flush(void);
  void Reset(void);
  
private:
  void AddCharToBuffer(uint8_t c);
  void AddCharToScreen(uint8_t c);
  
  void DrawBitmapToBuffer(int x, int y, const bool bitmap[24][18]);
  void DrawBitmapToScreen(int x, int y, const bool bitmap[24][18]);
  void DrawGrid(void);
  
  cMutex* osdMutex;
  cMutex* scrollerMutex;
  cMutex* bufferMutex;
  
  cOsd* osd;
  cBitmap* offScreenBuffer;
  
  Scroller* scroller;

#if VDRVERSNUM >= 10503 
  cFont* font; 
  cFont* fontNml;
  cFont* fontItl;
#else
  const cFont* font;
  const cFont* fontNml;
  const cFont* fontItl;
#endif
  
  int cRow;
  int cCol;
  int height;
  int osdWidth;// The actual osd width. will be a few pixels more than OSDWIDTH
  tColor bgClr;
  tColor fgClr;
  
  captionMode mode;
  fontMode   fmode;
};


///////////////////////////////////////////////////////////////////////////////


class Scroller
{
public:
  Scroller(int lines);
 ~Scroller();
 
 void AddChar(char c);
 char Get(int r, int c);
 int Lines(void) { return numLines; }
 void Scroll(void);
 void Backspace(void);
 void DER(void);
 void Reset(void);
 
private:
  void Put(int r, int c, char ch);
  void Print(void);
  char* data;
  int cCol;
  int numLines; 
};


///////////////////////////////////////////////////////////////////////////////


#endif //__NACC_OSD_H
