#include "setupmenu.h"
#include "common.h"


///////////////////////////////////////////////////////////////////////////////


static const char* const CCTypeStrings[4] = { "C1", "C2", "T1", "T2" };
static const char* const ColorStrings[4] = { "Black", "White", "Transparent", "Gray (50% Opaque)" };
static const char* const StartTypeStrings[3] = { "Automatic", "Manual", "On mute"};


///////////////////////////////////////////////////////////////////////////////


CCSetupMenu::CCSetupMenu(PluginCC* p, CCManager* m)
{
  plugin = p;
  manager = m;
  
  newCCType         = config.CCType;
  newActive         = config.Active;
  newOverrideColors = config.OverrideColors;
  newFgClr          = config.FgClr;
  newBgClr          = config.BgClr;
  newStartType      = config.StartType;
  newHideMenuEntry  = config.HideMenuEntry;

  ActiveItem = new cMenuEditBoolItem("Active", &newActive);
  CCTypeItem = NULL;
  OverrideColorsItem = NULL;
  BgClrItem = NULL;
  FgClrItem = NULL;
  StartTypeItem = NULL;
  HideMenuEntryItem = NULL;
  
  Add(ActiveItem);
  
  if (newActive) AddItems();

}


//-----------------------------------------------------------------------------

eOSState CCSetupMenu::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);
 
  cOsdItem* selected = Get(Current()); 
  
  switch (Key) {
    case kLeft:
    case kRight:
      if (selected  == OverrideColorsItem) {
        if (newOverrideColors && !BgClrItem) 
        {
          AddColorItems();
          Display();
        }
        else if (!newOverrideColors && BgClrItem)
        {
          DelColorItems();
          Display();
        }
      }
      else if (selected == StartTypeItem)
      {
        if (newStartType != S_AUTOMATIC && !HideMenuEntryItem) { 
          AddHideMenuItem();
          Display();
        }
        else if (newStartType == S_AUTOMATIC && HideMenuEntryItem) {
          DelHideMenuItem();
          Display();
        }
      }
      else if (selected == ActiveItem)
      {
        if (newActive && !CCTypeItem) {
          AddItems();
          Display();
        }
        else if (!newActive && CCTypeItem) {
          DelItems();
          Display();
        }
      }
      break;
    case kOk: 
      Store();
      state = osBack;
      break;  
    default: break;
  }
  
  return state;
}


//-----------------------------------------------------------------------------

void CCSetupMenu::AddItems(void) 
{
  CCTypeItem = new cMenuEditStraItem("  CC Type", &newCCType, 4, CCTypeStrings);
  StartTypeItem = new cMenuEditStraItem("  Start", &newStartType, 3, StartTypeStrings);
  
  Add(CCTypeItem);
  Add(StartTypeItem);
  
  if (newStartType != S_AUTOMATIC)
    AddHideMenuItem(); 
  
  OverrideColorsItem = new cMenuEditBoolItem("  Override Colors", &newOverrideColors);
  Add(OverrideColorsItem);  
  
  if (newOverrideColors)
    AddColorItems();
}


//-----------------------------------------------------------------------------

void CCSetupMenu::DelItems(void)
{
  Del(CCTypeItem->Index());    CCTypeItem = NULL;
  Del(StartTypeItem->Index()); StartTypeItem = NULL;
  
  if (newStartType != S_AUTOMATIC)
    DelHideMenuItem(); 
  
  Del(OverrideColorsItem->Index()); OverrideColorsItem = NULL; 
  
  if (newOverrideColors)
    DelColorItems();
}


//-----------------------------------------------------------------------------

void CCSetupMenu::AddHideMenuItem(void)
{
  HideMenuEntryItem = new cMenuEditBoolItem("    Hide Main Menu Entry", &newHideMenuEntry);
  Add(HideMenuEntryItem, false, StartTypeItem);
}


void CCSetupMenu::DelHideMenuItem(void)
{
  Del(HideMenuEntryItem->Index());
  HideMenuEntryItem = NULL;
}


//-----------------------------------------------------------------------------

void CCSetupMenu::AddColorItems(void)
{
  BgClrItem = new cMenuEditStraItem("    Background Color", &newBgClr, 4, ColorStrings);
  FgClrItem = new cMenuEditStraItem("    Forground Color",  &newFgClr, 4, ColorStrings);
  Add(BgClrItem, false, OverrideColorsItem);
  Add(FgClrItem, false, BgClrItem);
}


void CCSetupMenu::DelColorItems(void)
{
  Del(BgClrItem->Index()); BgClrItem = NULL;
  Del(FgClrItem->Index()); FgClrItem = NULL;
}


//-----------------------------------------------------------------------------

void CCSetupMenu::Store(void)
{
  bool callPlugin = (config.Active != newActive);
  bool colorsChanged = (config.BgClr != newBgClr) || (config.FgClr != newFgClr)
                       || (config.OverrideColors != newOverrideColors);
  bool startTypeChange = (newStartType != config.StartType);
  
  SetupStore("Active", config.Active = newActive);
  
  if (newActive)
  {
    SetupStore("CCType", config.CCType = newCCType);
    SetupStore("OverrideColors", config.OverrideColors = newOverrideColors);
    SetupStore("StartType", config.StartType = newStartType);
  
    if (newStartType != S_AUTOMATIC)
      SetupStore("HideMenuEntry", config.HideMenuEntry = newHideMenuEntry); 
  
    if (newOverrideColors) {
      SetupStore("BgClr",  config.BgClr = newBgClr);
      SetupStore("FgClr",  config.FgClr = newFgClr); 
    }
  }
     
  if (callPlugin) {
    dprint(L_MSG, "Setup   : Active has changed, %sing plugin.", newActive ? "start":"stopp");
    plugin->StartFromSetup(); 
  } 
  else if (startTypeChange)
  {
    if (newStartType == S_AUTOMATIC)
      plugin->StartFromSetup();
    else if (manager)
      manager->Stop();    
  }
  
  
  if (colorsChanged && manager) {
    if (newOverrideColors)
      manager->SetOsdColors(colors[newBgClr], colors[newFgClr]);
    else
      manager->SetOsdColors(clrBlack, clrWhite);  
  } 
  
  //TODO: Take some action if start type has changed 
}


///////////////////////////////////////////////////////////////////////////////

